disp('-------------------Eventevolution ver 0.9-------------------');

% Number of sampled DHTs for estimating one Rho
msample = 150;
%Random walk sample number
randomwalksample_num = 600;

max_hop = 12; % s in our approach
startyear = 1980;


% Read data in
% input files: people.txt, events.txt, socialnet.txt, occur.txt,
% interpolate_var.txt

if exist('data/people.txt','file') ~=2 || exist('data/events.txt','file') ~=2 || exist('data/socialnet.txt','file') ~=2 || exist('data/occur.txt','file') ~=2 || exist('data/interpolate_std.txt','file') ~=2
    disp('Necessary input files are missing! Put data files in folder data/!');
    return;
end

% EventsForTime.txt contains names of products (events) to investigate correlation
% evolution
if exist('data/nnzidx_cell.mat','file') ~=2 || exist('data/EventsForTime.txt','file') ~=2
    disp('data/nnzidx_cell.mat or data/EventsForTime.txt is missing!');
    return;
end

global sumofRowW nnzidx_cell;

disp('Reading data/nnzidx_cell.mat...');
load('data/nnzidx_cell.mat');

disp('Reading people.txt..');
people_col = textread('data/people.txt','%s');
n = size(people_col,1);
disp('Reading events.txt..');
product_col = textread('data/events.txt','%s');
r = size(product_col,1);

% Format: column 1: edge start person id, column 2: edge end person id,
% column 3: edge weight (current 1). For undirected graph, you should
% include both (p1, p2, 1) and (p2, p1, 1) in the file.
disp('Reading social network...');
[wcol1, wcol2, wcol3] = textread('data/socialnet.txt','%f%f%f');
disp('Done.');
disp('Constructing social network...');
W = sparse(wcol1,wcol2,wcol3,n,n);
sumofRowW = sum(W,2);
clear W wcol1 wcol2 wcol3;
disp('Done.');

% Format: column 1: product id, column 2: person id who bought the product,
% column 3: purchase time (milliseconds from 1970-01-01, in DBLP we use year)
disp('Reading occurring information of events...');
[rcol1, rcol2, rcol3] = textread('data/occur.txt','%f%f%f');
disp('Done.');
disp('Constructing corresponding matrix...');
%vals = ones(length(rcol1),1);
R = sparse(rcol1,rcol2,rcol3,r,n);
clear rcol1 rcol2 rcol3;
disp('Done.');

%Read products with which we want to investigate correlation changes in
%terms of time
EventsForTime = textread('data/EventsForTime.txt','%s');
tproductidx = zeros(1,length(EventsForTime));
tproductlength = 0;
for tpidx = 1 : length(EventsForTime)
    idxxx = strmatch(EventsForTime{tpidx}, product_col, 'exact');
    if ~isempty(idxxx)
        tproductidx(tpidx) = idxxx;
        tproductlength = tproductlength + 1;
    end
end

% This program can resume unaccomplished compuation if there is a
% evolution.mat in the output folder. If you want to start a new run,
% please remove output/evolution.mat.
if exist('output/evolution.mat','file') ==2
    load('output/evolution.mat');
else
    % the third dimension: first: upper bound, second: lower bound, 
    % third: heuristic mean, fourth: estimated std, fifth
    % : approximate z-score for upper bound, sixth row: approximate z-score 
    % for lower bound, seventh: mvalue
    significanceResults = zeros(tproductlength,2010-startyear+1,7);
    resumeitem = 1;
    resumeyear = startyear;
end

[mvals,randomstd] = textread('data/interpolate_std.txt','%f%f');

disp('Start computing correlations...');

s = RandStream.create('mt19937ar','seed',3);
RandStream.setDefaultStream(s);


% for each prodcut (event), estimate its averaged decay hitting time for
% each time step
for i = resumeitem : tproductlength
    productindex = tproductidx(i);
    whobuyit = full(R(productindex,:));
    whobuyit(whobuyit == 0) = inf;
    
    for yearss = resumeyear :  2010    
        buy_people = find(whobuyit <= yearss);
        if isempty(buy_people) || length(buy_people) == 1
            significanceResults(i,yearss-startyear+1,1) = 0;
            significanceResults(i,yearss-startyear+1,2) = 0;
            continue;
        end
        buynum = length(buy_people);
        significanceResults(i,yearss-startyear+1,7) = buynum;
        if buynum <= msample
            m = buynum;
        else
            disp(['There are ',num2str(buynum),' people who have event ',product_col{productindex},' for year ',num2str(yearss),'. Generating a sample of ',num2str(msample),' people to perform hitting simulation...']);
            m = msample;
            temppermutation = randperm(buynum);
            samplebuypeople = temppermutation(1:m);
        end
        currentm_results = zeros(2,m);
        % for each person who bought the product, compute his/her decay
        % hitting time bounds to other m-1 people
        for k = 1 : m
            if buynum <= msample
                disp(['Begin hitting simulatin for the ',num2str(k),'-th person who has event ',product_col{productindex},' for year ',num2str(yearss),'...']);
                startnode = buy_people(k);
                if k == 1
                    targets = buy_people(2:m);
                elseif k == m
                    targets = buy_people(1:(m-1));
                else
                    targets = [buy_people(1:(k-1)) buy_people((k+1):m)];
                end
            else
                disp(['Begin hitting simulatin for the ',num2str(k),'-th sampled person who has event ',product_col{productindex},' for year ',num2str(yearss),'...']);
                startnode = buy_people(samplebuypeople(k));
                if samplebuypeople(k) == 1
                    targets = buy_people(2:buynum);
                elseif samplebuypeople(k) == buynum
                    targets = buy_people(1:(buynum-1));
                else
                    targets = [buy_people(1:(samplebuypeople(k)-1)) buy_people((samplebuypeople(k)+1):buynum)];
                end
            end

            [currentm_results(1,k),currentm_results(2,k)] = SimulateDecayHitting(startnode,targets,max_hop,randomwalksample_num,n);
            if buynum <= msample
                disp(['Hitting simulatin for the ',num2str(k),'-th person done.']);
            else
                disp(['Hitting simulatin for the ',num2str(k),'-th sampled person done.']);
            end
        end

        significanceResults(i,yearss-startyear+1,1:2) = mean(currentm_results,2);
        disp(['Rho estimation for product (event) ',product_col{productindex},' and time ',num2str(yearss),' done.']);
   
        % heuristic mean
        p = (buynum-1)/(n-1);       
        expectedcorrelation = p / (1-exp(-1)*(1-p));      
        significanceResults(i,yearss-startyear+1,3) = expectedcorrelation;
        
        % estimate std
        if buynum < 2
                significanceResults(i,yearss-startyear+1,4) = 0;
        else
            largermidx = find(mvals > buynum,1);
            if isempty(largermidx) % larger than the last element of mvals
                significanceResults(i,yearss-startyear+1,4) = (randomstd(length(mvals)) * (n - buynum)) / (n - mvals(length(mvals)));

            elseif mvals(largermidx-1) == buynum
                significanceResults(i,yearss-startyear+1,4) = randomstd(largermidx-1);           
            else
                significanceResults(i,yearss-startyear+1,4) = randomstd(largermidx-1) + ( ((buynum-mvals(largermidx-1))*(randomstd(largermidx)-randomstd(largermidx-1)))/(mvals(largermidx)-mvals(largermidx-1)) );
            end
        end
        
        significanceResults(i,yearss-startyear+1,5) = (significanceResults(i,yearss-startyear+1,1) - significanceResults(i,yearss-startyear+1,3)) / significanceResults(i,yearss-startyear+1,4);
        significanceResults(i,yearss-startyear+1,6) = (significanceResults(i,yearss-startyear+1,2) - significanceResults(i,yearss-startyear+1,3)) / significanceResults(i,yearss-startyear+1,4);
        
        if yearss == 2010
            resumeitem = i + 1;
            resumeyear = startyear;
        else
            resumeitem = i;
            resumeyear = yearss + 1;
        end
        save('output/evolution.mat','significanceResults','resumeitem','resumeyear');
        disp(['Correlation estimation for product ',product_col{productindex},' and time ',num2str(yearss),' done.']);
    end
end

