disp('-------------------Rankevent ver 1.0-------------------');

% Number of sampled DHTs for estimating one Rho
msample = 150;
%Random walk sample number
randomwalksample_num = 600; 

max_hop = 9; % s in our approach


% Read data in
% input files: people.txt, events.txt, socialnet.txt, occur.txt,
% interpolate_var.txt
if exist('data/people.txt','file') ~=2 || exist('data/events.txt','file') ~=2 || exist('data/socialnet.txt','file') ~=2 || exist('data/occur.txt','file') ~=2 || exist('data/interpolate_std.txt','file') ~=2
    disp('Necessary input files are missing! Put data files in folder data/!');
    return;
end


if exist('data/nnzidx_cell.mat','file') ~=2
    disp('data/nnzidx_cell.mat are missing!');
    return;
end

global sumofRowW nnzidx_cell;

disp('Reading data/nnzidx_cell.mat...');
load('data/nnzidx_cell.mat');

disp('Reading people.txt..');
people_col = textread('data/people.txt','%s');
n = size(people_col,1);
disp('Reading events.txt..');
product_col = textread('data/events.txt','%s');
r = size(product_col,1);

% Format: column 1: edge start person id, column 2: edge end person id,
% column 3: edge weight (current 1). For undirected graph, you should
% include both (p1, p2, 1) and (p2, p1, 1) in the file.
disp('Reading social network...');
[wcol1, wcol2, wcol3] = textread('data/socialnet.txt','%f%f%f');
disp('Done.');
disp('Constructing social network...');
W = sparse(wcol1,wcol2,wcol3,n,n);
sumofRowW = sum(W,2);
clear W wcol1 wcol2 wcol3;
disp('Done.');

% Format: column 1: product id, column 2: person id who bought the product,
% column 3: purchase time (milliseconds from 1970-01-01)
disp('Reading occurring information of events...');
[rcol1, rcol2, rcol3] = textread('data/occur.txt','%f%f%f');
disp('Done.');
disp('Constructing corresponding matrix...');
vals = ones(length(rcol1),1);
R = sparse(rcol1,rcol2,vals,r,n);
sumofRowR = full(sum(R,2));
clear rcol1 rcol2 rcol3 vals;
disp('Done.');

[mvals,randomstd] = textread('data/interpolate_std.txt','%f%f');

disp('Start computing correlations...');

% Random seed for repeatability
s = RandStream.create('mt19937ar','seed',2);
RandStream.setDefaultStream(s);


% Store significance results (\tilde{\rho})
% first row: upper bound, second row: lower bound, third row: heuristic mean,
% fourth row: estimated std, fifth row: approximate z-score for upper bound, sixth row: 
% approximate z-score for lower bound
significanceResults = zeros(6,r);

% for each event (or product), estimate its averaged decay hitting time and
% estimated significance
for i = 1 : r
    buy_people = find(R(i,:));
    if isempty(buy_people) || length(buy_people) == 1
        significanceResults(1,i) = -Inf;
        significanceResults(2,i) = -Inf;
        continue;
    end
    buynum = length(buy_people);
    if buynum <= msample
        m = buynum;
    else
        disp(['There are ',num2str(buynum),' people who have event ',product_col{i},'. Generating a sample of ',num2str(msample),' people to perform hitting simulation...']);
        m = msample;
        temppermutation = randperm(buynum);
        samplebuypeople = temppermutation(1:m);
    end
    currentm_results = zeros(2,m);
    % for each person who bought the product, compute his/her decay
    % hitting time bounds to other m-1 people
    for k = 1 : m
        if buynum <= msample
            disp(['Begin hitting simulatin for the ',num2str(k),'-th person who have event ',product_col{i},'...']);
            startnode = buy_people(k);
            if k == 1
                targets = buy_people(2:m);
            elseif k == m
                targets = buy_people(1:(m-1));
            else
                targets = [buy_people(1:(k-1)) buy_people((k+1):m)];
            end
        else
            disp(['Begin hitting simulatin for the ',num2str(k),'-th sampled person who have event ',product_col{i},'...']);
            startnode = buy_people(samplebuypeople(k));
            if samplebuypeople(k) == 1
                targets = buy_people(2:buynum);
            elseif samplebuypeople(k) == buynum
                targets = buy_people(1:(buynum-1));
            else
                targets = [buy_people(1:(samplebuypeople(k)-1)) buy_people((samplebuypeople(k)+1):buynum)];
            end
        end
        
        [currentm_results(1,k),currentm_results(2,k)] = SimulateDecayHitting(startnode,targets,max_hop,randomwalksample_num,n);
        if buynum <= msample
            disp(['Hitting simulatin for the ',num2str(k),'-th person done.']);
        else
            disp(['Hitting simulatin for the ',num2str(k),'-th sampled person done.']);
        end
    end
    
    significanceResults(1:2,i) = mean(currentm_results,2);
    disp(['Rho estimation for product (event) ',product_col{i},' done.']);
    
    % heuristic mean
    p = (sumofRowR(i)-1)/(n-1);
    currexpect = p / (1-exp(-1)*(1-p));
    significanceResults(3,i) = currexpect;
    
    % estimate std
    if sumofRowR(i) < 2
        significanceResults(4,i) = 0;
    else
        largermidx = find(mvals>sumofRowR(i),1);
        if isempty(largermidx) % larger than the last element of mvals
            significanceResults(4,i) = (randomstd(length(mvals)) * (n - sumofRowR(i))) / (n - mvals(length(mvals)));

        elseif mvals(largermidx-1) == sumofRowR(i)
            significanceResults(4,i) = randomstd(largermidx-1);           
        else
            significanceResults(4,i) = randomstd(largermidx-1) + ( ((sumofRowR(i)-mvals(largermidx-1))*(randomstd(largermidx)-randomstd(largermidx-1)))/(mvals(largermidx)-mvals(largermidx-1)) );
        end
        %randomvar(i) = std(currentmsample_results_rw) / sqrt(sumofRowR(i));
    end  
    significanceResults(5,i) = (significanceResults(1,i) - significanceResults(3,i)) / significanceResults(4,i);
    significanceResults(6,i) = (significanceResults(2,i) - significanceResults(3,i)) / significanceResults(4,i);
    disp(['Correlation estimation for product ',product_col{i},' done.']);
end

finalranking = mean(significanceResults(5:6,:),1);

% Output a ranking list of products by correlation measure
[restrash, indexofproduct] = sort(finalranking,'descend');
fileoutput = fopen('output/rankingbysignificance.txt','w');
fprintf( fileoutput,'%s\t%s\t%s\t%s\t%s\t%s\t%s\r\n','name','(low+up)/2','low','up','mean','std','m');
for iiii = 1 : r
    fprintf( fileoutput,'%s\t%f\t%f\t%f\t%f\t%f\t%d\r\n',product_col{indexofproduct(iiii)}, finalranking(indexofproduct(iiii)), significanceResults(6,indexofproduct(iiii)), significanceResults(5,indexofproduct(iiii)), significanceResults(3,indexofproduct(iiii)), significanceResults(4,indexofproduct(iiii)), sumofRowR(indexofproduct(iiii)) );
end
fclose(fileoutput);