disp('-------------------Estimating variances for rho values of randomly selected m nodes-------------------');

% The number of samples in each run
randsample = 10000;
% To imporve efficiency, if mvalue > 5000, we generate 200 random DHTs (by changing the start node of hitting) from
% one C_n^mvalue permutation
msample = 200;
%Random walk sample number
randomwalksample_num = 800;

max_hop = 12; % s in our approach, see the paper

if exist('data/people.txt','file') ~=2 || exist('data/socialnet.txt','file') ~=2
    disp('Necessary input files are missing! Need data/people.txt and data/socialnet.txt');
    return;
end

% randomseeds.mat contains the seeds used to generate random sequences for variance
% estimations, colums correspond to mvalues. For each mvalue, we sample
% multiple times and take the averaged sample variance as the result. The
% rows represent those multiple samples. We found when mvalue > 5000, the
% sample variance is more stable than the other mvalues. When mvalue < 5, it is 
% also relatively stable. Therefore, we only take four samples for mvalue > 5000, 
% and ten samples for mvalues < 5
if exist('data/randomseeds.mat','file') ~=2
    disp('Necessary input files are missing! Need data/randomseeds.mat');
    return;
end

if exist('data/nnzidx_cell.mat','file') ~=2
    disp('data/nnzidx_cell.mat are missing!');
    return;
end

global sumofRowW nnzidx_cell;

disp('Reading data/nnzidx_cell.mat...');
load('data/nnzidx_cell.mat');

disp('Reading data/randomseeds.mat...');
load('data/randomseeds.mat');

mvalues = [2;20;50;100;200;500;1000;2000;5000;10000;30000;50000];


disp('Reading people.txt..');
people_col = textread('data/people.txt','%s');
n = size(people_col,1);


% Format: column 1: edge start person id, column 2: edge end person id,
% column 3: edge weight (current 1). For undirected graph, you should
% include both (p1, p2, 1) and (p2, p1, 1) in the file.
disp('Reading social network...');
[wcol1, wcol2, wcol3] = textread('data/socialnet.txt','%f%f%f');
disp('Done.');
disp('Constructing social network...');
W = sparse(wcol1,wcol2,wcol3,n,n);
sumofRowW = sum(W,2);
clear W wcol1 wcol2 wcol3;
disp('Done.');

% store all sample results, the last three columns only contain three
% results
results = zeros(size(randomseeds,1),length(mvalues));

% Since this loop is time-consuming, one can open several Matlab apps and do the
% iterations in parallel
for i = 1 : length(mvalues)
    if mvalues(i) > 5000
        snum = 4;
    elseif mvalues(i) < 5
        snum = 10;
    else
        snum = size(randomseeds,1);
    end
    for j = 1 : snum
        seed = randomseeds(j,i);
        s = RandStream.create('mt19937ar','seed',seed);
        RandStream.setDefaultStream(s);

        disp(['Start estimating variance for the ',num2str(j),'-th run of mvalue=',num2str(mvalues(i))]);

        currentmsample_results_rw = zeros(1,randsample);
    
        jjj = 1;
        while jjj <= randsample
            if mvalues(i) <= 5000
                disp(['Begin the ',num2str(jjj),'-th random DHT for ',num2str(mvalues(i)),' people. (run-',num2str(j),')']);
                tempsampling = randperm(n);
                buy_people = tempsampling(1:mvalues(i));
                %sampledm(samplednum,:) = buy_people;

                startnode = buy_people(1);
                targets = buy_people(2:length(buy_people));

                % Compute random walk simulation solution!!!!!!!!!!!!!!!!!!!!!

                %rwstart = tic;
                [uppper,lowwer] = SimulateDecayHitting(startnode,targets,max_hop,randomwalksample_num,n);
                currentmsample_results_rw(jjj) = (uppper+lowwer)/2;
                disp(['Random walk simulation for the ',num2str(jjj),'-th random DHT completed.']);
                jjj = jjj + 1;
                %samplednum = samplednum + 1;
            else
                tempsampling = randperm(n);
                buy_people = tempsampling(1:mvalues(i));
                %sampledm(samplednum,:) = buy_people;
                %samplednum = samplednum + 1;

                m = msample;
                temppermutation = randperm(mvalues(i));
                samplebuypeople = temppermutation(1:m);
                for k = 1 : m
                    if jjj > randsample
                        break;
                    end
                    disp(['Begin the ',num2str(jjj),'-th random DHT for ',num2str(mvalues(i)),' people. (run-',num2str(j),')']);
                    startnode = buy_people(samplebuypeople(k));
                    if samplebuypeople(k) == 1
                        targets = buy_people(2:mvalues(i));
                    elseif samplebuypeople(k) == mvalues(i)
                        targets = buy_people(1:(mvalues(i)-1));
                    else
                        targets = [buy_people(1:(samplebuypeople(k)-1)) buy_people((samplebuypeople(k)+1):mvalues(i))];
                    end
                    [uppper,lowwer] = SimulateDecayHitting(startnode,targets,max_hop,randomwalksample_num,n);
                    currentmsample_results_rw(jjj) = (uppper+lowwer)/2;
                    disp(['Random walk simulation for the ',num2str(jjj),'-th random DHT completed.']);
                    jjj = jjj + 1;
                end

            end
        end
    
        results(j,i) = std(currentmsample_results_rw) / sqrt(mvalues(i));
    end
    
end

finalresults = zeros(length(mvalues),1);
finalresults(1) = mean(results(1:10,1));
finalresults(2:9) = mean(results(:,2:9))';
finalresults(10:12) = mean(results(1:4,10:12))';

f = fopen('data/interpolate_std.txt','w');

for i = 1 : length(finalresults)
    fprintf(f,'%d',mvalues(i));
    fprintf(f,'\t%10.8f\r\n',finalresults(i));
end
fclose(f);
